--- chan_sip.c.orig	2005-07-22 20:40:03.703150912 -0600
+++ chan_sip.c	2005-07-22 21:32:56.871755960 -0600
@@ -139,6 +139,11 @@
 #define SIP_PUBLISH	14
 #define SIP_RESPONSE	100
 
+#define SUBSCRIBE_XPIDF_XML		1
+#define SUBSCRIBE_DIALOG_INFO_XML	2
+#define SUBSCRIBE_CPIM_PIDF_XML		3
+#define SUBSCRIBE_PIDF_XML		4
+
 #define RTP 	1
 #define NO_RTP	0
 const struct  cfsip_methods { 
@@ -4477,7 +4482,7 @@
 
 	reqprep(&req, p, SIP_NOTIFY, 0, 1);
 
-	if (p->subscribed == 1) {
+	if (p->subscribed == SUBSCRIBE_XPIDF_XML) {
 		ast_copy_string(to, get_header(&p->initreq, "To"), sizeof(to));
 
 		c = ditch_braces(to);
@@ -4528,7 +4533,7 @@
 		t += bytes;
 		maxbytes -= bytes;
 		bytes = snprintf(t, maxbytes, "</address>\n</atom>\n</presence>\n");
-	} else {
+	} else if (p->subscribed == SUBSCRIBE_DIALOG_INFO_XML) {
 		add_header(&req, "Event", "dialog");
 		add_header(&req, "Content-Type", "application/dialog-info+xml");
 
@@ -4547,6 +4552,9 @@
 		t += bytes;
 		maxbytes -= bytes;
 		bytes = snprintf(t, maxbytes, "</dialog>\n</dialog-info>\n");	
+	} else {
+		/* we somehow have a subscription that we don't know how to handle - ignore it */
+		return 0;
 	}
 	if (t > tmp + sizeof(tmp))
 		ast_log(LOG_WARNING, "Buffer overflow detected!!  (Please file a bug report)\n");
@@ -9409,11 +9417,28 @@
 				transmit_response(p, "484 Address Incomplete", req);
 			ast_set_flag(p, SIP_NEEDDESTROY);	
 		} else {
-			/* Initialize tag */	
-			p->tag = rand();
-			if (!strcmp(get_header(req, "Accept"), "application/dialog-info+xml"))
-				p->subscribed = 2;
-			else if (!strcmp(get_header(req, "Accept"), "application/simple-message-summary")) {
+			char *event = get_header(req, "Event"); /* Get Event package name */ /* Initialize tag */
+			char *accept = get_header(req, "Accept"); p->tag = rand();
+
+			if (!strcmp(event, "presence") || !strcmp(event, "dialog")) { /* Presence, RFC 3842 */ 
+				if (strstr(accept, "application/dialog-info+xml")) {
+					/* IETF draft: draft-ietf-sipping-dialog-package-05.txt */
+					/* Should not be used for SUBSCRIBE, but anyway */
+					p->subscribed = SUBSCRIBE_DIALOG_INFO_XML;
+				} else if (!p->subscribed && strstr(accept, "application/xpidf+xml")) {
+					p->subscribed = SUBSCRIBE_XPIDF_XML; /* Early pre-RFC 3863 format with MSN additions (Microsoft Messenger) */
+				} else if (!p->subscribed && strstr(p->useragent, "Polycom")) {
+					/* Polycom uses the presence event but does not provide an Accept header*/
+					p->subscribed = SUBSCRIBE_XPIDF_XML; /* Early pre-RFC 3863 format with MSN additions (Microsoft Messenger) */
+				} else {
+					/* Can't find a format for events that we know about */
+					transmit_response(p, "489 Bad Event", req);
+					if (option_debug > 1)
+						ast_log(LOG_DEBUG, "Received SIP subscribe for unknown accept format: %s\n", accept);
+					ast_set_flag(p, SIP_NEEDDESTROY);	
+					return 0;
+				}
+ 			} else if (!strcmp(event, "message-summary") && !strcmp(accept, "application/simple-message-summary")) {
 				/* Looks like they actually want a mailbox */
 
 				/* At this point, we should check if they subscribe to a mailbox that
@@ -9440,10 +9465,16 @@
 					ast_set_flag(p, SIP_NEEDDESTROY);	
 				}
 				return 0;
-			} else
-				p->subscribed = 1;
+			} else {
+				/* Can't find a format for events that we know about */
+				transmit_response(p, "489 Bad Event", req);
+				if (option_debug > 1)
+					ast_log(LOG_DEBUG, "Received SIP subscribe for unknown event package: %s\n", event);
+				ast_set_flag(p, SIP_NEEDDESTROY);	
+				return 0;
+			}
 			if (p->subscribed)
-				p->stateid = ast_extension_state_add(p->context, p->exten, cb_extensionstate, p);
+ 					p->stateid = ast_extension_state_add(p->context, p->exten, cb_extensionstate, p);
 		}
 	} else 
 		c = p->owner;
@@ -9457,7 +9488,7 @@
 			return 0;
 		}
 		/* The next line can be removed if the SNOM200 Expires bug is fixed */
-		if (p->subscribed == 1) {  
+		if (p->subscribed != 0) {  
 			if (p->expiry>max_expiry)
 				p->expiry = max_expiry;
 		}
